@@ -133,10 +133,25 @@ class FeedsView(hildon.GtkTreeView):
         dummy_feed_data.href = url
         dummy_feed_data.feed = feedparser.FeedParserDict()
         dummy_feed_data.feed.title = _('Retrieving info...')
-        store.set(feed_iter, 0, None, 1, dummy_feed_data.feed.title, 2, dummy_feed_data, 3, -1)
+        store.set(feed_iter, self.FEEDS_ICON_COLUMN, None,
+                  self.FEEDS_NAME_COLUMN, self.get_feed_title_markup(dummy_feed_data),
+                  self.FEEDS_DATA_COLUMN, dummy_feed_data,
+                  self.FEEDS_READ_COLUMN, 0)
 
         return (dummy_feed_data,feed_iter)
 
+    def get_feed_title_markup(self, feed_data):
+        if 'link' in  feed_data.feed:
+            url = feed_data.feed.link
+        else:
+            url = feed_data.href
+
+        color = self.style.lookup_color(constants.SECONDARY_TEXT_COLOR).to_string()
+        return '%s\n<span face="monospace" foreground="%s"' \
+            'size="x-small">%s</span>' % \
+            (feed_data.feed.title, color, url)
+
+
 class PreferencesDialog(gtk.Dialog):
 
     def __init__(self, parent):
@@ -208,11 +223,11 @@ class FeedsWindow(hildon.StackableWindow):
             os.mkdir(constants.RSS_CONF_FOLDER, 0700)
             os.mkdir(os.path.join(constants.RSS_CONF_FOLDER, 'icons'), 0700)
 
-        # Load Feeds (could be none)
-        try:
-            manager.load(self._feed_data_loaded)
-        except IOError:
-            pass
+        # Feeds
+        self.view = FeedsView(gtk.HILDON_UI_MODE_NORMAL)
+        self.view.set_model (gtk.ListStore(gtk.gdk.Pixbuf, str, gobject.TYPE_PYOBJECT, int))
+        self.view.connect ("row-activated", self._on_feed_activated)
+        self.view.show()
 
         # Used for osso context in rotation object
         app_name = constants.RSS_NAME
@@ -227,12 +242,6 @@ class FeedsWindow(hildon.StackableWindow):
         self.edit_toolbar = hildon.EditToolbar()
         self.edit_toolbar.connect('arrow-clicked', self._restore_normal_mode)
 
-        # Feeds
-        self.view = FeedsView(gtk.HILDON_UI_MODE_NORMAL)
-        self.view.set_model (gtk.ListStore(gtk.gdk.Pixbuf, str, gobject.TYPE_PYOBJECT, int))
-        self.view.connect ("row-activated", self._on_feed_activated)
-        self.view.show()
-
         align = gtk.Alignment(0, 0, 1, 1)
         align.set_padding(4, 0 , 16, 16)
 
@@ -250,6 +259,13 @@ class FeedsWindow(hildon.StackableWindow):
         self.connect('delete-event', self._on_exit)
         hildon.Program.get_instance().add_window(self)
 
+        # Load Feeds (could be none)
+        try:
+            manager.load(self._feed_data_loaded)
+        except IOError:
+            pass
+
+
     def _on_feed_activated(self, treeview, path, column):
         feed_iter = self.view.get_model().get_iter(path)
         feed_data = self.view.get_model().get_value(feed_iter, self.view.FEEDS_DATA_COLUMN)
@@ -340,6 +356,8 @@ class FeedsWindow(hildon.StackableWindow):
         if not paths:
             hildon.hildon_banner_show_information(self, '', _('Please select at least one Feed'))
             return
+
+        removed = []
         for path in paths:
             try:
                 manager.remove_feed(model[path][FeedsView.FEEDS_DATA_COLUMN])
@@ -347,7 +365,10 @@ class FeedsWindow(hildon.StackableWindow):
                 print 'Could not remove', constants.RSS_DB_FILE
                 hildon.hildon_banner_show_information(self, '', _('File error while removing Feed'))
             else:
-                model.remove(model.get_iter(path))
+                removed.append(gtk.TreeRowReference(model, path))
+
+        for reference in removed:
+            model.remove(model.get_iter(reference.get_path()))
 
         # restore normal mode
         self._restore_normal_mode(button)
@@ -395,17 +416,6 @@ class FeedsWindow(hildon.StackableWindow):
 
             dialog.destroy()
 
-    def _get_feed_title_markup(self, feed_data):
-        if 'link' in  feed_data.feed:
-            url = feed_data.feed.link
-        else:
-            url = feed_data.href
-
-        color = self.style.lookup_color(constants.SECONDARY_TEXT_COLOR).to_string()
-        return '%s\n<span face="monospace" foreground="%s"' \
-            'size="x-small">%s</span>' % \
-            (feed_data.feed.title, color, url)
-
     def _feed_added_cb(self, pixbuf, new_feed_data, row_reference):
 
         # Remove progress information
@@ -423,8 +433,10 @@ class FeedsWindow(hildon.StackableWindow):
             return
 
         # Update model
-        store.set(feed_iter, 0, pixbuf, 1, self._get_feed_title_markup(new_feed_data),
-                  2, new_feed_data, 3, len(new_feed_data.entries))
+        store.set(feed_iter, self.view.FEEDS_ICON_COLUMN, pixbuf,
+                  self.view.FEEDS_NAME_COLUMN, self.view.get_feed_title_markup(new_feed_data),
+                  self.view.FEEDS_DATA_COLUMN, new_feed_data,
+                  self.view.FEEDS_READ_COLUMN, len(new_feed_data.entries))
 
         # Take screenshot
         hildon.hildon_gtk_window_take_screenshot(self, False)
@@ -449,8 +461,10 @@ class FeedsWindow(hildon.StackableWindow):
                 iter = store.append()
                 pixbuf = manager.get_favicon(feed_data.feed.link or feed_data.href)
                 unread_count = len([entry for entry in feed_data.entries if entry.read == False])
-                store.set(iter, 0, pixbuf, 1, self._get_feed_title_markup(feed_data),
-                          2, feed_data, 3, unread_count)
+                store.set(iter, self.view.FEEDS_ICON_COLUMN, pixbuf,
+                          self.view.FEEDS_NAME_COLUMN, self.view.get_feed_title_markup(feed_data),
+                          self.view.FEEDS_DATA_COLUMN, feed_data,
+                          self.view.FEEDS_READ_COLUMN, unread_count)
                 # Improve responsiveness
 #                while gtk.events_pending():
 #                    gtk.main_iteration_do()
@@ -672,6 +686,9 @@ class ItemWindowHeader(gtk.HBox):
         print 'Open ' + link_button.get_uri()
 
     def set_item(self, item):
+
+        self.item = item
+
         # Mark item as read
         item['read'] = True
 
@@ -847,3 +864,14 @@ class ItemWindow(hildon.StackableWindow):
         self.document.open_stream(content_type)
         self.document.write_stream("<html><body>%s</body></html>" % body)
         self.document.close_stream()
+
+        # Update button sensitiviness
+        path = self.model.get_path(self.model_iter)
+        position = path[-1]
+        if position == 0:
+            self.item_window_header.button_up.set_sensitive(False)
+        elif position == len(self.model) - 1:
+            self.item_window_header.button_down.set_sensitive(False)
+        else:
+            self.item_window_header.button_up.set_sensitive(True)
+            self.item_window_header.button_down.set_sensitive(True)
